#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2018-08-02
# 
# Monitor status of Check Point internal ca (ica)
#
# 2019-09-23: parameter discovery
# 2020-06-08: changed snmp-scan function
# 2021-08-11: rewritten for CMK 2.0
# 2023-05-29: moved gui files to ~/local/lib/check_mk/gui/plugins/...
#
# sample snmpwalk
#
# .1.3.6.1.4.1.2620.1.19.1.0 = Hex-STRING: 43 50 20 49 6E 74 65 72 6E 61 6C 20 43 41 00
# .1.3.6.1.4.1.2620.1.19.2.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.19.3.0 = STRING: "26"
# .1.3.6.1.4.1.2620.1.19.4.0 = STRING: "0"
# .1.3.6.1.4.1.2620.1.19.5.0 = STRING: "6"
# .1.3.6.1.4.1.2620.1.19.6.0 = STRING: "0"
# .1.3.6.1.4.1.2620.1.19.7.0 = STRING: "20"
# .1.3.6.1.4.1.2620.1.19.8.0 = STRING: "0"
# .1.3.6.1.4.1.2620.1.19.9.0 = STRING: "0"
# .1.3.6.1.4.1.2620.1.19.10.0 = STRING: "0"
# .1.3.6.1.4.1.2620.1.19.11.0 = STRING: "0"
# .1.3.6.1.4.1.2620.1.19.12.0 = STRING: "18"
# .1.3.6.1.4.1.2620.1.19.13.0 = STRING: "8"
# .1.3.6.1.4.1.2620.1.19.14.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.19.15.0 = Gauge32: 991140056
#
#

# sample string_table
#
# [
#  [
#   '1', '6', '0', '6', '0', '0', '0', '2', '0', '2', '4', '0', '1', '996000058'
#  ]
# ]
#
# sample section
#
# CheckpointIca(icaStatus='1', icaTotalNumberOfCertificates=6, icaPendingCertificates=0, icaValidCertificates=6,
#               icaRenewedCertificates=0, icarevokedCertificates=0, icaExpieredCertificates=0,
#               icaTotalCertificatesOfUsers=2, icaCertificatesOfInternalUsers=0, icaCertificatesOfLdapUsers=2,
#               icaTotalNumberOfSicCertificates=4, icaTotalNumberOfIkeCertificates=0, icaLastCrlDistributionPoint='1',
#               icaBuild='996000058'
#               )
#

from typing import Optional
from dataclasses import dataclass

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    equals,
    Result,
    State,
    SNMPTree,
    startswith,
    all_of,
    any_of,
    check_levels,
)


@dataclass
class CheckpointIca:
    icaStatus: int
    icaTotalNumberOfCertificates: int
    icaPendingCertificates: int
    icaValidCertificates: int
    icaRenewedCertificates: int
    icarevokedCertificates: int
    icaExpieredCertificates: int
    icaTotalCertificatesOfUsers: int
    icaCertificatesOfInternalUsers: int
    icaCertificatesOfLdapUsers: int
    icaTotalNumberOfSicCertificates: int
    icaTotalNumberOfIkeCertificates: int
    icaLastCrlDistributionPoint: int
    icaBuild: str


def parse_checkpoint_ica(string_table: StringTable) -> Optional[CheckpointIca]:
    try:
        icaStatus, icaTotalNumberOfCertificates, icaPendingCertificates, icaValidCertificates, \
        icaRenewedCertificates, icarevokedCertificates, icaExpieredCertificates, icaTotalCertificatesOfUsers, \
        icaCertificatesOfInternalUsers, icaCertificatesOfLdapUsers, icaTotalNumberOfSicCertificates, \
        icaTotalNumberOfIkeCertificates, icaLastCrlDistributionPoint, icaBuild = string_table[0]
    except (IndexError, ValueError):
        return

    return CheckpointIca(
        icaStatus=int(icaStatus),
        icaTotalNumberOfCertificates=int(icaTotalNumberOfCertificates),
        icaPendingCertificates=int(icaPendingCertificates),
        icaValidCertificates=int(icaValidCertificates),
        icaRenewedCertificates=int(icaRenewedCertificates),
        icarevokedCertificates=int(icarevokedCertificates),
        icaExpieredCertificates=int(icaExpieredCertificates),
        icaTotalCertificatesOfUsers=int(icaTotalCertificatesOfUsers),
        icaCertificatesOfInternalUsers=int(icaCertificatesOfInternalUsers),
        icaCertificatesOfLdapUsers=int(icaCertificatesOfLdapUsers),
        icaTotalNumberOfSicCertificates=int(icaTotalNumberOfSicCertificates),
        icaTotalNumberOfIkeCertificates=int(icaTotalNumberOfIkeCertificates),
        icaLastCrlDistributionPoint=int(icaLastCrlDistributionPoint),
        icaBuild=icaBuild,
    )


def discovery_checkpoint_ica(section: CheckpointIca) -> DiscoveryResult:
    yield Service()


def check_checkpoint_ica(section: CheckpointIca):
    if section.icaStatus == 1:
        yield Result(state=State.OK, summary=f'Status: OK')
    else:
        yield Result(state=State.CRIT, summary=f'Status: not OK')
    yield Result(state=State.OK, summary=f'Build: {section.icaBuild}')

    for label, value, metric in [
        ('Total number of certificates', section.icaTotalNumberOfCertificates, 'icatotalnumberofcertificates'),
        ('Valid certificates', section.icaValidCertificates, 'icavalidcertificates'),
        ('Pending certificates', section.icaPendingCertificates, 'icapendingcertificates'),
        ('Revoked certificates', section.icarevokedCertificates, 'icarevokedcertificates'),
        ('Total certificates of users', section.icaTotalCertificatesOfUsers, 'icatotalcertificatesofusers'),
        ('Certificates of LDAP users', section.icaCertificatesOfLdapUsers, 'icacertificatesofldapusers'),
        ('Certificates of internal users', section.icaCertificatesOfInternalUsers, 'icacertificatesofinternalusers'),
        ('Total number of SIC certificates', section.icaTotalNumberOfSicCertificates, 'icatotalnumberofsiccertificates'),
        ('Total number of IKE certificates', section.icaTotalNumberOfIkeCertificates, 'icatotalnumberofikecertificates'),
        ('Renewed Certificates', section.icaRenewedCertificates, 'icarenewedcertificates'),
        ('Expired Certificates', section.icaExpieredCertificates, 'icaexpieredcertificates'),
        ('Last Crl Distribution Point', section.icaLastCrlDistributionPoint, 'icaLastCrlDistributionPoint'),
    ]:
        yield from check_levels(
            value=value,
            label=label,
            metric_name=f'checkpoint_ica_{metric}',
            notice_only=True,
            boundaries=(0, None),
            render_func= lambda v: f'{v:.0f}',
        )


register.snmp_section(
    name='checkpoint_ica',
    parse_function=parse_checkpoint_ica,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.1.19',  # CHECKPOINT-MIB::products.19 (not described in MIB file :-( )
        oids=[
            '2',  # ica_2  # Status
            '3',  # ica_3  # TotalNumberOfCertificates
            '4',  # ica_4  # PendingCertificates
            '5',  # ica_5  # ValidCertificates
            '6',  # ica_6  # ExpieredCertificates|RenewedCertificates
            '7',  # ica_7  # revokedCertificates
            '8',  # ica_8  # ExpieredCertificates|RenewedCertificates
            '9',  # ica_9  # TotalCertificatesOfUsers
            '10',  # ica_10 # CertificatesOfInternalUsers
            '11',  # ica_11 # CertificatesOfLdapUsers
            '12',  # ica_12 # TotalNumberOfSicCertificates
            '13',  # ica_13 # TotalNumberOfIkeCertificates
            '14',  # ica_14 # LastCrlDistributionPoint
            '15',  # icaBuild
        ]),
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_ica',
    service_name='Internal CA',
    discovery_function=discovery_checkpoint_ica,
    check_function=check_checkpoint_ica,
)
